@@ -1,14 +1,17 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+import inspect
 import os
-from astropy.coordinates import SkyCoord
-from astropy.coordinates.representation import UnitSphericalRepresentation
+
+from astropy.coordinates import (BaseCoordinateFrame, frame_transform_graph,
+                                 SkyCoord, UnitSphericalRepresentation)
 
 from .core import (nside_to_pixel_area, nside_to_pixel_resolution,
                    nside_to_level, nside_to_npix, npix_to_nside,
                    healpix_to_lonlat, lonlat_to_healpix,
+                   healpix_to_xyz, xyz_to_healpix,
                    bilinear_interpolation_weights, interpolate_bilinear_lonlat,
                    ring_to_nested, nested_to_ring, healpix_cone_search,
-                   boundaries_lonlat, neighbours)
+                   boundaries_lonlat, neighbours, _validate_order)
 from .utils import parse_input_healpix_data
 
 __all__ = ['HEALPix']
@@ -26,6 +29,36 @@ class NoFrameError(Exception):
         super().__init__(NO_FRAME_MESSAGE.format(alternative_method))
 
 
+def _get_frame(frame):
+    """
+    Get a frame instance or None, from the input `frame`, which could be a
+    frame name string, frame instance, or frame class.
+
+    Adapted from
+    :meth:`astropy.coordinates.sky_coordinate_parsers._get_frame_class`.
+    """
+
+    if frame is None or isinstance(frame, BaseCoordinateFrame):
+        return frame
+
+    elif isinstance(frame, str):
+        frame_cls = frame_transform_graph.lookup_name(frame)
+        if frame_cls is None:
+            frame_names = frame_transform_graph.get_names()
+            raise ValueError('Coordinate frame name "{}" is not a known '
+                             'coordinate frame ({})'
+                             .format(frame, sorted(frame_names)))
+        return frame_cls()
+
+    elif inspect.isclass(frame) and issubclass(frame, BaseCoordinateFrame):
+        return frame()
+
+    else:
+        raise ValueError("Coordinate frame must be a frame name, frame "
+                         "instance, frame class, or None, not a '{}'"
+                         .format(frame.__class__.__name__))
+
+
 class HEALPix:
     """
     A HEALPix pixellization.
@@ -34,22 +67,29 @@ class HEALPix:
     ----------
     nside : int
         Number of pixels along the side of each of the 12 top-level HEALPix tiles
-    order : { 'nested' | 'ring' }
-        Order of HEALPix pixels
-    frame : :class:`~astropy.coordinates.BaseCoordinateFrame`, optional
+    order : { 'nested' | 'ring' }, optional
+        Order of HEALPix pixels. Input string can be lower or upper case.
+    frame : str or :class:`~astropy.coordinates.BaseCoordinateFrame`, optional
         The celestial coordinate frame of the pixellization. This can be
         ommitted, in which case the pixellization will not be attached to any
         particular celestial frame, and the methods ending in _skycoord will
         not work (but the _lonlat methods will still work and continue to
-        return generic longitudes/latitudes).
+        return generic longitudes/latitudes). The frame may be passed as a
+        string (such as ``galactic``), as a frame class, or as an instance of
+        a frame class.
+
+    Raises
+    ------
+    ValueError
+        If 'order' is not one of the allowed options.
     """
 
     def __init__(self, nside=None, order='ring', frame=None):
         if nside is None:
             raise ValueError('nside has not been set')
         self.nside = nside
-        self.order = order
-        self.frame = frame
+        self.order = _validate_order(order)
+        self.frame = _get_frame(frame)
 
     @classmethod
     def from_header(cls, input_data, field=0, hdu_in=None, nested=None):
@@ -161,6 +201,59 @@ class HEALPix:
         return lonlat_to_healpix(lon, lat, self.nside,
                                  return_offsets=return_offsets, order=self.order)
 
+    def healpix_to_xyz(self, healpix_index, dx=None, dy=None):
+        """
+        Convert HEALPix indices (optionally with offsets) to Cartesian coordinates
+
+        Parameters
+        ----------
+        healpix_index : `~numpy.ndarray`
+            1-D array of HEALPix indices
+        dx, dy : `~numpy.ndarray`, optional
+            1-D arrays of offsets inside the HEALPix pixel, which must be in
+            the range [0:1] (0.5 is the center of the HEALPix pixels). If not
+            specified, the position at the center of the pixel is used.
+
+        Returns
+        -------
+        x : :class:`~numpy.ndarray`
+            The x coordinates
+        y : :class:`~numpy.ndarray`
+            The y coordinates
+        z : :class:`~numpy.ndarray`
+            The z coordinates
+        """
+        return healpix_to_xyz(healpix_index, self.nside, dx=dx, dy=dy, order=self.order)
+
+    def xyz_to_healpix(self, x, y, z, return_offsets=False):
+        """
+        Convert Cartesian coordinates to HEALPix indices (optionally with offsets)
+
+        Parameters
+        ----------
+        x : :class:`~numpy.ndarray`
+            The x coordinates
+        y : :class:`~numpy.ndarray`
+            The y coordinates
+        z : :class:`~numpy.ndarray`
+            The z coordinates
+        return_offsets : bool
+            If `True`, the returned values are the HEALPix pixel as well as
+            ``dx`` and ``dy``, the fractional positions inside the pixel. If
+            `False` (the default), only the HEALPix pixel is returned.
+
+        Returns
+        -------
+        healpix_index : `~numpy.ndarray`
+            1-D array of HEALPix indices
+        dx, dy : `~numpy.ndarray`
+            1-D arrays of offsets inside the HEALPix pixel in the range [0:1] (0.5
+            is the center of the HEALPix pixels). This is returned if
+            ``return_offsets`` is `True`.
+        """
+        return xyz_to_healpix(x, y, z, self.nside,
+                              return_offsets=return_offsets, order=self.order)
+
     def nested_to_ring(self, nested_index):
         """
         Convert a healpix 'nested' index to a healpix 'ring' index
